from pathlib import Path

from haplo.data_preparation import constantinos_kalapotharakos_format_file_to_sqlite


def example_data_preparation():
    constantinos_kalapotharakos_format_file_to_sqlite(
        Path('data/mcmc_vac_all_800k.dat'), Path('data/800k_parameters_and_phase_amplitudes.db'))


if __name__ == '__main__':
    example_data_preparation()



import re

from pathlib import Path

from haplo.train_logging_configuration import TrainLoggingConfiguration


def test_passing_no_session_directory_to_new_creates_one_based_on_the_datetime():
    configuration = TrainLoggingConfiguration.new()
    match = re.search(r'\d{4}_\d{2}_\d{2}_\d{2}_\d{2}_\d{2}', configuration.session_directory.name)
    assert match is not None


def test_passing_specific_session_directory_to_new():
    session_directory = Path('a')
    configuration = TrainLoggingConfiguration.new(session_directory=session_directory)
    assert configuration.session_directory == Path('a')



import _thread
import datetime
import logging
import os
import re
import subprocess
import threading

logger = logging.getLogger(__name__)


def schedule_self_process_interrupt_signal_before_pbs_end_time(
        time_before_pbs_end_time: datetime.timedelta = datetime.timedelta(minutes=1)) -> None:
    if os.environ.get('PBS_JOBID') is None:
        return

    def kill_process():
        logger.info(f'Sending self terminate signal before PBS wall time limit is reached. '
                    f'Current time is {datetime.datetime.now(tz=datetime.timezone.utc)} UTC.')
        _thread.interrupt_main()

    completed_process = subprocess.run(['qstat', '-f', os.environ['PBS_JOBID']], capture_output=True, text=True)
    process_output = completed_process.stdout
    pbs_start_timestamp_match = re.search(r'\n[^\S\r\n]*stime[^\S\r\n]*=[^\S\r\n]*(\d+)[^\S\r\n]+\(', process_output)
    pbs_start_timestamp = int(pbs_start_timestamp_match.group(1))
    pbs_start_time = datetime.datetime.fromtimestamp(pbs_start_timestamp, tz=datetime.timezone.utc)
    pbs_wall_time_match = re.search(
        r'\n[^\S\r\n]*Resource_List\.walltime[^\S\r\n]*=[^\S\r\n]*(\d+):(\d+):(\d+)\n',
        process_output)
    pbs_wall_time_hours = int(pbs_wall_time_match.group(1))
    pbs_wall_time_minutes = int(pbs_wall_time_match.group(2))
    pbs_wall_time_seconds = int(pbs_wall_time_match.group(3))
    pbs_wall_time_delta = datetime.timedelta(hours=pbs_wall_time_hours, minutes=pbs_wall_time_minutes,
                                             seconds=pbs_wall_time_seconds)
    pbs_end_time = pbs_start_time + pbs_wall_time_delta
    process_kill_datetime = pbs_end_time - time_before_pbs_end_time
    delay = process_kill_datetime - datetime.datetime.now(tz=datetime.timezone.utc)
    logger.info(f'PBS end time: {pbs_end_time} UTC.')
    logger.info(f'Self kill scheduled end time: {process_kill_datetime} UTC.')
    threading.Timer(delay.total_seconds(), kill_process).start()



from __future__ import annotations

import datetime
import os
from dataclasses import dataclass
from pathlib import Path
from typing import Any, Dict


@dataclass
class TrainLoggingConfiguration:
    """
    Configuration settings for the logging of a train session.

    :ivar wandb_project: The wandb project to log to.
    :ivar wandb_entity: The wandb entity to log to.
    :ivar additional_log_dictionary: The dictionary of additional values to log.
    """
    wandb_project: str
    wandb_entity: str
    additional_log_dictionary: Dict[str, Any]
    session_directory: Path
    model_save_cycle_frequency: int | None = None

    @classmethod
    def new(cls,
            wandb_project: str = 'ramjet',
            wandb_entity: str = 'example',
            additional_log_dictionary: Dict[str, Any] | None = None,
            session_directory: Path | None = None,
            model_save_cycle_frequency: int | None = None):
        if additional_log_dictionary is None:
            additional_log_dictionary = {}
        session_directory_environment_variable = os.environ.get('HAPLO_SESSION_DIRECTORY')
        if session_directory_environment_variable is not None and session_directory is not None:
            raise ValueError(f'Passing a `session_directory` is not allowed when the environment variable '
                             f'`HAPLO_SESSION_DIRECTORY` is set.')
        if session_directory is None:
            session_directory = session_directory_environment_variable
        if session_directory is None:
            datetime_string = datetime.datetime.now().strftime("%Y_%m_%d_%H_%M_%S")
            session_directory = Path(f'sessions/{datetime_string}')
        if isinstance(session_directory, str):
            session_directory = Path(session_directory)
        return cls(wandb_project=wandb_project,
                   wandb_entity=wandb_entity,
                   additional_log_dictionary=additional_log_dictionary,
                   session_directory=session_directory,
                   model_save_cycle_frequency=model_save_cycle_frequency)



import numpy as np
import os
import torch
from pathlib import Path

from haplo.export_onnx import WrappedModel
from haplo.models import SingleDenseNetwork
from haplo.nicer_dataset import NicerDataset, split_dataset_into_count_datasets
from haplo.nicer_transform import PrecomputedNormalizeParameters, PrecomputedNormalizePhaseAmplitudes


def test_infer_session():
    os.environ["WANDB_MODE"] = "disabled"
    full_dataset_path = Path(__file__).parent.joinpath('test_train_session_resources/300_parameters_and_phase_amplitudes.db')
    full_train_dataset = NicerDataset.new(
        dataset_path=full_dataset_path,
        length=300,
        parameters_transform=PrecomputedNormalizeParameters(),
        phase_amplitudes_transform=PrecomputedNormalizePhaseAmplitudes(),
        in_memory=True
    )
    test_dataset, validation_dataset, train_dataset, _ = split_dataset_into_count_datasets(
        full_train_dataset, [10, 10, 100])

    model = SingleDenseNetwork()
    model = WrappedModel(model)  # The DDP module requires an extra wrapping. This emulates that.
    saved_model_path = Path(__file__).parent.joinpath('test_infer_session_resources/test_infer_session_model.pt')
    model.load_state_dict(torch.load(str(saved_model_path), map_location=torch.device('cpu')))
    model.eval()

    test_parameters0, test_phase_amplitudes0 = test_dataset[0]
    input_array = np.expand_dims(test_parameters0, axis=0)
    with torch.no_grad():
        input_tensor = torch.tensor(input_array)
        output_tensor = model(input_tensor)
        output_array = output_tensor.numpy()
    predicted_test_phase_amplitudes0 = np.squeeze(output_array, axis=0)
    assert predicted_test_phase_amplitudes0.shape == (64,)



import getpass
import logging
import math
import os
import socket
from pathlib import Path
from typing import Callable, List

import numpy as np
import stringcase
import torch
import wandb as wandb
from torch import Tensor, tensor
from torch.distributed import init_process_group, destroy_process_group, ReduceOp
from torch.nn import Module
from torch.nn.parallel import DistributedDataParallel
from torch.nn.utils import clip_grad_norm_
from torch.optim import Optimizer
from torch.types import Device
from torch.utils.data import DataLoader, DistributedSampler, Dataset

from haplo.distributed import ddp_setup
from haplo.logging import set_up_default_logger
from haplo.losses import norm_based_gradient_clip
from haplo.nicer_dataset import nicer_dataset_worker_initialization_function, disconnect, \
    move_sqlite_subset_to_new_file, NicerDataset
from haplo.rank_constant_distributed_sampler import RankConstantDistributedSampler
from haplo.train_hyperparameter_configuration import TrainHyperparameterConfiguration
from haplo.train_logging_configuration import TrainLoggingConfiguration
from haplo.train_system_configuration import TrainSystemConfiguration
from haplo.unwrap_model import unwrap_model
from haplo.wandb_liaison import wandb_init, wandb_log, wandb_commit, \
    wandb_log_dictionary, wandb_log_data_class, wandb_save_manual_config_file

logger = logging.getLogger(__name__)
non_blocking = True  # TODO: Should probably be elsewhere.
pin_memory = True  # TODO: Should probably be elsewhere.


def train_session(train_dataset: Dataset, validation_dataset: Dataset, model: Module, loss_function: Module,
                  metric_functions: List[Module], optimizer: Optimizer,
                  hyperparameter_configuration: TrainHyperparameterConfiguration,
                  system_configuration: TrainSystemConfiguration,
                  logging_configuration: TrainLoggingConfiguration):
    torch.multiprocessing.set_start_method('spawn')
    ddp_setup(system_configuration)
    set_up_default_logger()
    logger.info(f'Host: {socket.gethostname()}')
    logger.info('Starting training...')
    logging_configuration.session_directory.mkdir(exist_ok=True, parents=True)
    local_rank, process_rank, world_size = get_distributed_world_information()
    wandb_init(process_rank=process_rank, project=logging_configuration.wandb_project,
               entity=logging_configuration.wandb_entity, settings=wandb.Settings(start_method='fork'))
    wandb_log_data_class(hyperparameter_configuration, process_rank=process_rank)
    wandb_log_data_class(system_configuration, process_rank=process_rank)
    wandb_log_dictionary(logging_configuration.additional_log_dictionary, process_rank=process_rank)
    log_distributed_settings(hyperparameter_configuration, system_configuration, process_rank)
    logger.info(wandb.config)
    wandb_save_manual_config_file(process_rank)

    network_device = get_device(local_rank)
    loss_function = loss_function.to(network_device)
    metric_functions = [metric_function.to(network_device) for metric_function in metric_functions]

    model = distribute_model_across_devices(model, optimizer, network_device)

    logger.info(f'{process_rank}: Loading dataset...')
    optimizer.zero_grad()
    train_dataloader, validation_dataloader = create_data_loaders(train_dataset, validation_dataset,
                                                                  hyperparameter_configuration.batch_size,
                                                                  system_configuration)
    sessions_directory = Path('sessions')
    sessions_directory.mkdir(parents=True, exist_ok=True)

    train_loop(model, train_dataloader, validation_dataloader, optimizer, loss_function, metric_functions,
               hyperparameter_configuration.cycles, network_device, process_rank, world_size,
               logging_configuration)

    destroy_process_group()


def log_distributed_settings(hyperparameter_configuration: TrainHyperparameterConfiguration,
                             system_configuration: TrainSystemConfiguration, process_rank: int) -> None:
    training_processes = int(os.environ.get('WORLD_SIZE'))
    training_processes_per_node = int(os.environ.get('LOCAL_WORLD_SIZE'))
    wandb_log_dictionary(
        {
            'training_processes': training_processes,
            'training_processes_per_node': training_processes_per_node,
            'nodes': training_processes // training_processes_per_node,
            'preprocessing_processes': training_processes * system_configuration.preprocessing_processes_per_train_process,
            'global_batch_size': training_processes * hyperparameter_configuration.batch_size
        },
        process_rank=process_rank)


def distribute_model_across_devices(model, optimizer, device):
    model = model.to(device, non_blocking=non_blocking)
    optimizer_to_device(optimizer, device)
    if torch.cuda.is_available():
        logger.info(f'Number of CUDA devices found on host: {torch.cuda.device_count()}')
        logger.info(f'Device: {device}')
        torch.cuda.set_device(device)
        model = DistributedDataParallel(model, device_ids=[device])
    else:
        logger.info(f'Device: cpu')
        model = DistributedDataParallel(model)
    return model


def train_loop(model, train_dataloader, validation_dataloader, optimizer, loss_function, metric_functions,
               cycles_to_run, network_device, process_rank, world_size,
               logging_configuration: TrainLoggingConfiguration):
    lowest_validation_cycle_loss = tensor(math.inf)
    logger.info(f'{process_rank}: Starting training loop...')
    for cycle in range(cycles_to_run):
        logger.info(f"Epoch {cycle} -------------------------------")
        train_phase(train_dataloader, model, loss_function, optimizer, network_device=network_device,
                    cycle=cycle, metric_functions=metric_functions, process_rank=process_rank,
                    world_size=world_size)
        validation_cycle_loss = validation_phase(validation_dataloader, model, loss_function,
                                                 network_device=network_device,
                                                 cycle=cycle,
                                                 metric_functions=metric_functions,
                                                 process_rank=process_rank, world_size=world_size)
        save_state(model, optimizer, logging_configuration, state_name_prefix='latest', process_rank=process_rank)
        if (
                logging_configuration.model_save_cycle_frequency is not None and
                cycle != 0 and
                cycle % logging_configuration.model_save_cycle_frequency == 0
        ):
            save_state(model, optimizer, logging_configuration, state_name_prefix=f'cycle_{cycle}',
                       process_rank=process_rank)
        if validation_cycle_loss < lowest_validation_cycle_loss:
            lowest_validation_cycle_loss = validation_cycle_loss
            save_state(model, optimizer, logging_configuration, state_name_prefix='lowest_validation',
                       process_rank=process_rank)
        wandb_log('epoch', cycle, process_rank=process_rank)
        wandb_log('cycle', cycle, process_rank=process_rank)
        wandb_commit(process_rank=process_rank)
    logger.info("Done!")


def get_device(local_rank):
    if torch.cuda.is_available():
        network_device = torch.device(f'cuda:{local_rank}')
    else:
        network_device = torch.device('cpu')
    return network_device


def get_distributed_world_information():
    process_rank = int(os.environ['RANK'])
    local_rank = int(os.environ['LOCAL_RANK'])
    world_size = int(os.environ['WORLD_SIZE'])
    return local_rank, process_rank, world_size


def create_data_loaders(train_dataset, validation_dataset, batch_size_per_device,
                        system_configuration: TrainSystemConfiguration):
    if system_configuration.preprocessing_processes_per_train_process > 0:
        prefetch_factor = 10
        persistent_workers = True
    else:
        prefetch_factor = None
        persistent_workers = False
    # TODO: Hacked way of passing samplers to init functions.
    train_sampler = RankConstantDistributedSampler(train_dataset)
    validation_sampler = RankConstantDistributedSampler(validation_dataset)
    if issubclass(NicerDataset, type(train_dataset.dataset)):
        if train_dataset.dataset.in_memory:
            # TODO: Hacked way of moving dataset to tmp.
            logger.info(f'Start of in memory branch...')
            local_rank, process_rank, world_size = get_distributed_world_information()
            train_indexes = list(iter(train_sampler))
            validation_indexes = list(iter(validation_sampler))
            train_subset_indexes = np.array(train_dataset.indices)[train_indexes].tolist()
            validation_subset_indexes = np.array(validation_dataset.indices)[validation_indexes].tolist()
            indexes = train_subset_indexes + validation_subset_indexes
            logger.info(f'Number of indexes on process: {len(indexes)}')
            # TODO: This hardcoded path changes depending on system.
            local_directory = Path(f'/tmp/{getpass.getuser()}')
            local_directory.mkdir(parents=True, exist_ok=True)
            local_database_path = local_directory.joinpath(f'qusi_{process_rank}.db')
            local_database_path.unlink(missing_ok=True)
            move_sqlite_subset_to_new_file(train_dataset.dataset.database_path, local_database_path, indexes)
            database_path = local_database_path
            database_uri = f'sqlite:///{database_path}?mode=ro'
            train_dataset.dataset.database_path = database_path
            validation_dataset.dataset.database_path = database_path
            train_dataset.dataset.database_uri = database_uri
            validation_dataset.dataset.database_uri = database_uri
            disconnect(train_dataset.dataset)
            disconnect(validation_dataset.dataset)
        worker_init_fn = nicer_dataset_worker_initialization_function
    else:
        worker_init_fn = None
    logger.info(f'Creating train data loader...')
    train_dataloader = DataLoader(train_dataset, batch_size=batch_size_per_device,
                                  num_workers=system_configuration.preprocessing_processes_per_train_process,
                                  pin_memory=pin_memory, persistent_workers=persistent_workers,
                                  prefetch_factor=prefetch_factor, shuffle=False,
                                  sampler=train_sampler,
                                  worker_init_fn=worker_init_fn)
    logger.info(f'Creating validation data loader...')
    validation_dataloader = DataLoader(validation_dataset, batch_size=batch_size_per_device,
                                       num_workers=system_configuration.preprocessing_processes_per_train_process,
                                       pin_memory=pin_memory, persistent_workers=persistent_workers,
                                       prefetch_factor=prefetch_factor, shuffle=False,
                                       sampler=validation_sampler,
                                       worker_init_fn=worker_init_fn)
    logger.info(f'Data loaders created.')
    torch.distributed.monitored_barrier(timeout=system_configuration.data_loader_creation_barrier_timeout)
    return train_dataloader, validation_dataloader


def save_state(model: Module, optimizer: Optimizer, logging_configuration: TrainLoggingConfiguration,
               state_name_prefix: str, process_rank: int):
    if process_rank == 0:
        torch.save(model.state_dict(),
                   logging_configuration.session_directory.joinpath(f'{state_name_prefix}_model.pt'))
        torch.save(optimizer.state_dict(), logging_configuration.session_directory.joinpath(
            f'{state_name_prefix}_optimizer.pt'))


def load_latest_state(model: Module, optimizer: Optimizer, session_directory: Path):
    model.load_state_dict(unwrap_model(torch.load(session_directory.joinpath('latest_model.pt'), map_location='cpu')))
    optimizer.load_state_dict(
        unwrap_model(torch.load(session_directory.joinpath('latest_optimizer.pt'), map_location='cpu')))


def train_phase(dataloader: DataLoader, model: Module, loss_function: Callable[[Tensor, Tensor], Tensor],
                optimizer: Optimizer, network_device: Device, cycle: int,
                metric_functions: List[Callable[[Tensor, Tensor], Tensor]], process_rank: int, world_size: int):
    model.train()
    total_cycle_loss = tensor(0, dtype=torch.float32, device='cpu', requires_grad=False)
    metric_totals = torch.zeros(size=[len(metric_functions)], device='cpu', requires_grad=False)
    assert (isinstance(dataloader.sampler, DistributedSampler) or
            isinstance(dataloader.sampler, RankConstantDistributedSampler))
    dataloader.sampler.set_epoch(cycle)
    batch_count = 0
    for batch, (parameters, light_curves) in enumerate(dataloader):
        parameters = parameters.to(network_device, non_blocking=non_blocking)
        light_curves = light_curves.to(network_device, non_blocking=non_blocking)
        predicted_light_curves = model(parameters)
        loss, total_cycle_loss = record_metrics(predicted_light_curves, light_curves, loss_function, total_cycle_loss,
                                                metric_functions, metric_totals)
        optimizer.zero_grad()
        loss_on_loss_device = loss.to(network_device, non_blocking=non_blocking)
        loss_on_loss_device.backward()
        apply_norm_based_gradient_clip_to_all_parameters(model)
        optimizer.step()

        if batch % 100 == 0:
            current = (batch + 1) * len(parameters)
            logger.info(f"loss: {loss.item():>7f}  [{current:>5d}/{len(dataloader.sampler):>5d}]")
        batch_count += 1
    log_metrics(total_cycle_loss, metric_functions, metric_totals, '', batch_count, world_size, process_rank)


def record_metrics(predicted_light_curves, light_curves, loss_function, total_cycle_loss, metric_functions,
                   metric_totals):
    loss = loss_function(predicted_light_curves, light_curves)
    total_cycle_loss += loss.item()
    for metric_function_index, metric_function in enumerate(metric_functions):
        batch_metric_value = metric_function(predicted_light_curves.detach(), light_curves)
        metric_totals[metric_function_index] += batch_metric_value.item()
    return loss, total_cycle_loss


def validation_phase(dataloader: DataLoader, model: Module, loss_function: Callable[[Tensor, Tensor], Tensor],
                     network_device: Device, cycle: int,
                     metric_functions: List[Callable[[Tensor, Tensor], Tensor]], process_rank: int, world_size: int
                     ) -> float:
    total_cycle_loss = tensor(0, dtype=torch.float32, device='cpu', requires_grad=False)
    metric_totals = torch.zeros(size=[len(metric_functions)], device='cpu', requires_grad=False)
    model.eval()
    assert (isinstance(dataloader.sampler, DistributedSampler) or
            isinstance(dataloader.sampler, RankConstantDistributedSampler))
    dataloader.sampler.set_epoch(cycle)
    with torch.no_grad():
        batch_count = 0
        for parameters, light_curves in dataloader:
            parameters = parameters.to(network_device, non_blocking=non_blocking)
            light_curves = light_curves.to(network_device, non_blocking=non_blocking)
            predicted_light_curves = model(parameters)
            loss, total_cycle_loss = record_metrics(predicted_light_curves, light_curves, loss_function,
                                                    total_cycle_loss,
                                                    metric_functions, metric_totals)
            batch_count += 1

    cycle_loss = log_metrics(total_cycle_loss, metric_functions, metric_totals, 'val_', batch_count, world_size,
                             process_rank)
    return cycle_loss


def log_metrics(total_cycle_loss: Tensor, metric_functions: List[Callable[[Tensor, Tensor], Tensor]],
                metric_totals: Tensor, prefix: str, number_of_batches: int, world_size: int, process_rank: int
                ) -> float:
    cycle_loss = total_cycle_loss / number_of_batches
    torch.distributed.reduce(cycle_loss, dst=0, op=ReduceOp.SUM)
    cycle_loss /= world_size
    wandb_log(f'{prefix}loss', cycle_loss, process_rank=process_rank)
    cycle_metric_values = metric_totals / number_of_batches
    torch.distributed.reduce(cycle_metric_values, dst=0, op=ReduceOp.SUM)
    cycle_metric_values /= world_size
    for metric_function_index, metric_function in enumerate(metric_functions):
        cycle_metric_value = cycle_metric_values[metric_function_index]
        wandb_log(f'{prefix}{get_metric_name(metric_function)}', cycle_metric_value,
                  process_rank=process_rank)
    return cycle_loss


def get_metric_name(metric_function):
    metric_name = type(metric_function).__name__
    metric_name = stringcase.snakecase(metric_name)
    metric_name = metric_name.replace('_metric', '').replace('_loss', '')
    return metric_name


def add_norm_based_gradient_clip_to_all_parameters(model):
    for parameter in model.parameters():
        parameter.register_hook(norm_based_gradient_clip)


def apply_norm_based_gradient_clip_to_all_parameters(model):
    for parameter in model.parameters():
        clip_grad_norm_(parameter, 1.0)


def optimizer_to_device(optimizer, device):
    for param in optimizer.state.values():
        if isinstance(param, torch.Tensor):
            param.data = param.data.to(device, non_blocking=non_blocking)
            if param._grad is not None:
                param._grad.data = param._grad.data.to(device, non_blocking=non_blocking)
        elif isinstance(param, dict):
            for subparam in param.values():
                if isinstance(subparam, torch.Tensor):
                    subparam.data = subparam.data.to(device, non_blocking=non_blocking)
                    if subparam._grad is not None:
                        subparam._grad.data = subparam._grad.data.to(device, non_blocking=non_blocking)



from pathlib import Path

import onnx
import torch
from onnxruntime.tools.onnx_model_utils import make_dim_param_fixed
from torch.nn import Module

from haplo.models import Cura


class WrappedModel(Module):
    def __init__(self, module):
        super(WrappedModel, self).__init__()
        self.module = module

    def forward(self, x):
        return self.module(x)


def export_onnx(model: Module, output_onnx_model_path: Path, input_features: int = 11):
    model.eval()
    fake_input = torch.randn(1, input_features, requires_grad=True)
    _ = model(fake_input)  # Model must be run to trace.
    torch.onnx.export(model,
                      fake_input,
                      str(output_onnx_model_path),
                      export_params=True,
                      opset_version=11,
                      do_constant_folding=True,
                      input_names=['input'],
                      output_names=['output'],
                      dynamic_axes={'output': {0: 'batch_size', 1: 'phase_amplitudes_length'}},
                      )
    onnx_model = onnx.load(str(output_onnx_model_path))
    make_dim_param_fixed(onnx_model.graph, 'batch_size', 1)
    make_dim_param_fixed(onnx_model.graph, 'phase_amplitudes_length', 64)
    onnx.save(onnx_model, str(output_onnx_model_path))


def export_onnx_model_from_pytorch_path(pytorch_path: Path):
    model = Cura()
    model = WrappedModel(model)
    model.load_state_dict(torch.load(pytorch_path, map_location='cpu'))
    export_onnx(model, output_onnx_model_path=Path('exported_model.onnx'))


if __name__ == '__main__':
    export_onnx_model_from_pytorch_path(Path('sessions/1.pt'))



import pandas as pd
import sqlite3

def check_640m():
    # Read sqlite query results into a pandas DataFrame
    # con = sqlite3.connect("data/640m_parameters_and_phase_amplitudes.db")
    sqliteConnection = sqlite3.connect("data_nobackup/640m_parameters_and_phase_amplitudes.db")
    sql_query = """SELECT name FROM sqlite_master 
       WHERE type='table';"""

    # Creating cursor object using connection object
    cursor = sqliteConnection.cursor()

    # executing our sql query
    cursor.execute(sql_query)
    print("List of tables\n")

    # printing all tables list
    print(cursor.fetchall())

    cursor.execute("SELECT COUNT(*) FROM main")
    result = cursor.fetchone()[0]
    print(result)

    # Execute the PRAGMA integrity_check command
    cursor.execute("PRAGMA integrity_check;")
    result = cursor.fetchall()
    print(result)




    #  df = pd.read_sql_query("SELECT *", con)

    # Verify that result of SQL query is stored in the dataframe
    #  print(df)
    # print(len(df))

    sqliteConnection.close()

if __name__ == '__main__':
    check_640m()


import os
import re
import shutil
import socket
from pathlib import Path

from filelock import FileLock

def move_path_to_nvme(path: Path) -> Path:
    match = re.match(r"gpu\d{3}", socket.gethostname())
    if match is not None:
        nvme_path = Path("/lscratch/golmsche").joinpath(path)
        if not nvme_path.exists():
            nvme_path.parent.mkdir(exist_ok=True, parents=True)
            nvme_lock_path = nvme_path.parent.joinpath(nvme_path.name + '.lock')
            lock = FileLock(str(nvme_lock_path))
            with lock.acquire():
                if not nvme_path.exists():
                    nvme_tmp_path = nvme_path.parent.joinpath(nvme_path.name + '.tmp')
                    shutil.copy(path, nvme_tmp_path)
                    nvme_tmp_path.rename(nvme_path)
        return nvme_path
    else:
        return path


def move_to_tmp_on_pbs(path: Path) -> Path:
    if 'PBS_JOBID' in os.environ:
        new_path = Path("/tmp").joinpath(path)
        if not new_path.exists():
            new_path.parent.mkdir(exist_ok=True, parents=True)
            nvme_lock_path = new_path.parent.joinpath(new_path.name + '.lock')
            lock = FileLock(str(nvme_lock_path))
            with lock.acquire():
                if not new_path.exists():
                    nvme_tmp_path = new_path.parent.joinpath(new_path.name + '.tmp')
                    shutil.copy(path, nvme_tmp_path)
                    nvme_tmp_path.rename(new_path)
        return new_path
    else:
        return path

rotated_dataset_path = Path('data/640m_rotated_parameters_and_phase_amplitudes.arrow')
constantinos_kalapotharakos_format_rotated_dataset_path = Path('data/mcmc_vac_all_640m_A.dat')
unrotated_dataset_path = Path('data/800k_parameters_and_phase_amplitudes.arrow')
constantinos_kalapotharakos_format_unrotated_dataset_path = Path('data/mcmc_vac_all_800k.dat')


import logging
from dataclasses import fields, dataclass
from pathlib import Path
from typing import Any, Dict

import wandb as wandb
import yaml

logger = logging.getLogger(__name__)


def wandb_log(name: str, value: Any, process_rank: int):
    if process_rank == 0:  # Only log for the first process.
        wandb.log({name: value}, commit=False)


def wandb_commit(process_rank: int):
    if process_rank == 0:  # Only log for the first process.
        wandb.log({}, commit=True)


def wandb_set_run_name(run_name: str, process_rank: int):
    if process_rank == 0:  # Only log for the first process.
        wandb.run.notes = run_name


def wandb_init(process_rank: int, **kwargs):
    if process_rank == 0:  # Only log for the first process.
        logger.info(f'{process_rank}: Starting wandb...')
        wandb.init(**kwargs)


def wandb_log_hyperparameter(name: str, value: Any, process_rank: int):
    if process_rank == 0:  # Only log for the first process.
        wandb.config[name] = value


def wandb_log_dictionary(log_dictionary: Dict[str, Any], process_rank: int):
    for key, value in log_dictionary.items():
        wandb_log_hyperparameter(key, value, process_rank)


def wandb_log_data_class(data_class: dataclass, process_rank: int):
    for field in fields(data_class):
        wandb_log_hyperparameter(field.name, getattr(data_class, field.name), process_rank)


def wandb_save_file(path: Path, process_rank: int):
    if process_rank == 0:  # Only log for the first process.
        wandb.save(str(path))


def wandb_save_manual_config_file(process_rank):
    if process_rank == 0:
        manual_config_path = Path(wandb.run.dir).joinpath('manual_config.yaml')
        with manual_config_path.open('w') as manual_config_file:
            yaml.dump(wandb.config, manual_config_file)
        wandb_save_file(manual_config_path, process_rank=process_rank)



import os
import tempfile
from pathlib import Path
from torch.optim import AdamW

from haplo.losses import SumDifferenceSquaredOverMedianExpectedSquaredMetric, PlusOneChiSquaredStatisticMetric, \
    PlusOneBeforeUnnormalizationChiSquaredStatisticMetric
from haplo.models import SingleDenseNetwork
from haplo.nicer_dataset import NicerDataset, split_dataset_into_count_datasets
from haplo.nicer_transform import PrecomputedNormalizeParameters, PrecomputedNormalizePhaseAmplitudes
from haplo.train_hyperparameter_configuration import TrainHyperparameterConfiguration
from haplo.train_logging_configuration import TrainLoggingConfiguration
from haplo.train_session import train_session
from haplo.train_system_configuration import TrainSystemConfiguration


def test_simple_train_session():
    os.environ["WANDB_MODE"] = "disabled"
    full_dataset_path = Path(__file__).parent.joinpath(
        'test_train_session_resources/300_parameters_and_phase_amplitudes.db')
    full_train_dataset = NicerDataset.new(
        dataset_path=full_dataset_path,
        length=300,
        parameters_transform=PrecomputedNormalizeParameters(),
        phase_amplitudes_transform=PrecomputedNormalizePhaseAmplitudes(),
        in_memory=True
    )
    test_dataset, validation_dataset, train_dataset, _ = split_dataset_into_count_datasets(
        full_train_dataset, [10, 10, 100])
    model = SingleDenseNetwork()
    loss_function = SumDifferenceSquaredOverMedianExpectedSquaredMetric()
    metric_functions = [PlusOneChiSquaredStatisticMetric(), PlusOneBeforeUnnormalizationChiSquaredStatisticMetric(),
                        SumDifferenceSquaredOverMedianExpectedSquaredMetric()]
    hyperparameter_configuration = TrainHyperparameterConfiguration.new(cycles=5, batch_size=50)
    system_configuration = TrainSystemConfiguration.new(preprocessing_processes_per_train_process=0)
    optimizer = AdamW(params=model.parameters(), lr=hyperparameter_configuration.learning_rate,
                      weight_decay=hyperparameter_configuration.weight_decay,
                      eps=hyperparameter_configuration.optimizer_epsilon)
    run_comments = 'run_comments_placeholder'  # Whatever you want to log in a string.
    additional_log_dictionary = {
        'model_name': type(model).__name__, 'train_dataset_size': len(train_dataset), 'run_comments': run_comments
    }
    logging_configuration = TrainLoggingConfiguration.new(
        wandb_project='test', wandb_entity='test', additional_log_dictionary=additional_log_dictionary,
        session_directory=Path(tempfile.gettempdir()))
    train_session(train_dataset=train_dataset, validation_dataset=validation_dataset, model=model,
                  loss_function=loss_function, metric_functions=metric_functions, optimizer=optimizer,
                  hyperparameter_configuration=hyperparameter_configuration, system_configuration=system_configuration,
                  logging_configuration=logging_configuration)



import os


def main():
    rank = os.environ.get('RANK')
    haplo_session_directory = os.environ.get('HAPLO_SESSION_DIRECTORY')
    print(f'{rank}: {haplo_session_directory}')


if __name__ == '__main__':
    main()



import datetime

import torch
import torch.distributed as dist
import torch.nn as nn
import torch.optim as optim

from torch.nn.parallel import DistributedDataParallel as DDP

class ToyModel(nn.Module):
    def __init__(self):
        super(ToyModel, self).__init__()
        self.net1 = nn.Linear(10, 10)
        self.relu = nn.ReLU()
        self.net2 = nn.Linear(10, 5)

    def forward(self, x):
        return self.net2(self.relu(self.net1(x)))


def demo_basic():
    dist.init_process_group("gloo")
    rank = dist.get_rank()
    print(f"Start running basic DDP example on rank {rank}.")

    device_id = rank % torch.cuda.device_count()
    model = ToyModel().to(device_id)
    ddp_model = DDP(model, device_ids=[device_id])

    loss_fn = nn.MSELoss()
    optimizer = optim.SGD(ddp_model.parameters(), lr=0.001)

    optimizer.zero_grad()
    outputs = ddp_model(torch.randn(20, 10))
    labels = torch.randn(20, 5).to(device_id)
    loss_fn(outputs, labels).backward()
    optimizer.step()
    torch.distributed.monitored_barrier(timeout=datetime.timedelta(minutes=5))

    optimizer.zero_grad()
    outputs = ddp_model(torch.randn(20, 10))
    labels = torch.randn(20, 5).to(device_id)
    loss_fn(outputs, labels).backward()
    optimizer.step()
    torch.distributed.monitored_barrier(timeout=datetime.timedelta(minutes=5))

    dist.destroy_process_group()

if __name__ == "__main__":
    demo_basic()


from datetime import timedelta

from pathlib import Path

from torch.optim import AdamW

from haplo.distributed import distributed_logging
from haplo.losses import PlusOneBeforeUnnormalizationChiSquaredStatisticMetric, \
    PlusOneChiSquaredStatisticMetric, SumDifferenceSquaredOverMedianExpectedSquaredMetric
from haplo.models import Cura
from haplo.nicer_dataset import NicerDataset, split_dataset_into_count_datasets
from haplo.nicer_transform import PrecomputedNormalizeParameters, PrecomputedNormalizePhaseAmplitudes
from haplo.pbs_helper import schedule_self_process_interrupt_signal_before_pbs_end_time
from haplo.train_hyperparameter_configuration import TrainHyperparameterConfiguration
from haplo.train_logging_configuration import TrainLoggingConfiguration
from haplo.train_session import train_session, load_latest_state
from haplo.train_system_configuration import TrainSystemConfiguration


@distributed_logging
def example_train_session():
    full_dataset_path = Path('data/800k_parameters_and_phase_amplitudes.db')
    full_train_dataset = NicerDataset.new(
        dataset_path=full_dataset_path,
        length=600_000_000,
        parameters_transform=PrecomputedNormalizeParameters(),
        phase_amplitudes_transform=PrecomputedNormalizePhaseAmplitudes(),
        in_memory=True
    )
    test_dataset, validation_dataset, train_dataset, _ = split_dataset_into_count_datasets(
        full_train_dataset, [10_000, 10_000, 50_000])
    model = Cura()
    loss_function = SumDifferenceSquaredOverMedianExpectedSquaredMetric()
    metric_functions = [PlusOneChiSquaredStatisticMetric(), PlusOneBeforeUnnormalizationChiSquaredStatisticMetric(),
                        SumDifferenceSquaredOverMedianExpectedSquaredMetric()]
    hyperparameter_configuration = TrainHyperparameterConfiguration.new(cycles=5)
    system_configuration = TrainSystemConfiguration.new(preprocessing_processes_per_train_process=1,
                                                        data_loader_creation_barrier_timeout=timedelta(hours=2))
    optimizer = AdamW(params=model.parameters(), lr=hyperparameter_configuration.learning_rate,
                      weight_decay=hyperparameter_configuration.weight_decay,
                      eps=hyperparameter_configuration.optimizer_epsilon)
    load_latest_state(model, optimizer, Path('sessions/2024_03_03_10_12_25_32_node_2_gpus_1_pp_500m_qi_loss_continued'))
    run_comments = f'base_old_version'  # Whatever you want to log in a string.
    additional_log_dictionary = {
        'model_name': type(model).__name__, 'train_dataset_size': len(train_dataset), 'run_comments': run_comments
    }
    logging_configuration = TrainLoggingConfiguration.new(
        wandb_project='profiling', wandb_entity='ramjet', additional_log_dictionary=additional_log_dictionary)
    # schedule_self_process_interrupt_signal_before_pbs_end_time()
    train_session(train_dataset=train_dataset, validation_dataset=validation_dataset, model=model,
                  loss_function=loss_function, metric_functions=metric_functions, optimizer=optimizer,
                  hyperparameter_configuration=hyperparameter_configuration, system_configuration=system_configuration,
                  logging_configuration=logging_configuration)


if __name__ == '__main__':
    example_train_session()
